@@ -22,7 +22,7 @@
 #include "libavcodec/vorbisdsp.h"
 
 void ff_vorbis_inverse_coupling_neon(float *mag, float *ang,
-                                     intptr_t blocksize);
+                                     ptrdiff_t blocksize);
 
 av_cold void ff_vorbisdsp_init_aarch64(VorbisDSPContext *c)
 {
