@@ -70,7 +70,6 @@ static av_cold int cri_decode_init(AVCodecContext *avctx)
         return AVERROR(ENOMEM);
     s->jpeg_avctx->flags = avctx->flags;
     s->jpeg_avctx->flags2 = avctx->flags2;
-    s->jpeg_avctx->dct_algo = avctx->dct_algo;
     s->jpeg_avctx->idct_algo = avctx->idct_algo;
     ret = avcodec_open2(s->jpeg_avctx, codec, NULL);
     if (ret < 0)
@@ -221,12 +220,10 @@ static int cri_decode_frame(AVCodecContext *avctx, AVFrame *p,
             if (bytestream2_get_le32(gb) != 0)
                 return AVERROR_INVALIDDATA;
             break;
-        case 102:;
-            int read_len = FFMIN(length, sizeof(codec_name) - 1);
-            if (read_len != bytestream2_get_buffer(gb, codec_name, read_len))
-                return AVERROR_INVALIDDATA;
-            length -= read_len;
-            if (strncmp(codec_name, "cintel_craw", read_len))
+        case 102:
+            bytestream2_get_buffer(gb, codec_name, FFMIN(length, sizeof(codec_name) - 1));
+            length -= FFMIN(length, sizeof(codec_name) - 1);
+            if (strncmp(codec_name, "cintel_craw", FFMIN(length, sizeof(codec_name) - 1)))
                 return AVERROR_INVALIDDATA;
             compressed = 1;
             goto skip;
@@ -405,17 +402,14 @@ skip:
     }
 
     if (hflip || vflip) {
-        rotation = av_frame_new_side_data(p, AV_FRAME_DATA_DISPLAYMATRIX,
-                                          sizeof(int32_t) * 9);
+        ff_frame_new_side_data(avctx, p, AV_FRAME_DATA_DISPLAYMATRIX,
+                               sizeof(int32_t) * 9, &rotation);
         if (rotation) {
             av_display_rotation_set((int32_t *)rotation->data, 0.f);
             av_display_matrix_flip((int32_t *)rotation->data, hflip, vflip);
         }
     }
 
-    p->pict_type = AV_PICTURE_TYPE_I;
-    p->flags |= AV_FRAME_FLAG_KEY;
-
     *got_frame = 1;
 
     return 0;
