@@ -23,7 +23,7 @@
 #include <celt/celt_header.h>
 #include "avcodec.h"
 #include "codec_internal.h"
-#include "internal.h"
+#include "decode.h"
 #include "libavutil/intreadwrite.h"
 
 struct libcelt_context {
@@ -129,11 +129,12 @@ static int libcelt_dec_decode(AVCodecContext *c, AVFrame *frame,
 
 const FFCodec ff_libcelt_decoder = {
     .p.name         = "libcelt",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("Xiph CELT decoder using libcelt"),
+    CODEC_LONG_NAME("Xiph CELT decoder using libcelt"),
     .p.type         = AVMEDIA_TYPE_AUDIO,
     .p.id           = AV_CODEC_ID_CELT,
     .p.capabilities = AV_CODEC_CAP_DR1,
     .p.wrapper_name = "libcelt",
+    .caps_internal  = FF_CODEC_CAP_NOT_INIT_THREADSAFE,
     .priv_data_size = sizeof(struct libcelt_context),
     .init           = libcelt_dec_init,
     .close          = libcelt_dec_close,
