@@ -25,8 +25,8 @@
  */
 
 #include "libavutil/buffer.h"
+#include "libavutil/mem.h"
 
-#include "config.h"
 #include "codec_internal.h"
 #include "encode.h"
 #include "libwebpenc_common.h"
@@ -168,6 +168,7 @@ const FFCodec ff_libwebp_anim_encoder = {
     .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_DELAY |
                       AV_CODEC_CAP_ENCODER_REORDERED_OPAQUE,
     .p.pix_fmts     = ff_libwebpenc_pix_fmts,
+    .color_ranges   = AVCOL_RANGE_MPEG,
     .p.priv_class   = &ff_libwebpenc_class,
     .p.wrapper_name = "libwebp",
     .caps_internal  = FF_CODEC_CAP_NOT_INIT_THREADSAFE,
