@@ -26,9 +26,9 @@
  * @see http://bespin.org/~qz/pc-gpe/pcx.txt
  */
 
+#include "libavutil/imgutils_internal.h"
 #include "avcodec.h"
 #include "bytestream.h"
-#include "libavutil/imgutils.h"
 #include "codec_internal.h"
 #include "encode.h"
 
@@ -194,9 +194,10 @@ static int pcx_encode_frame(AVCodecContext *avctx, AVPacket *pkt,
 
 const FFCodec ff_pcx_encoder = {
     .p.name         = "pcx",
-    .p.long_name    = NULL_IF_CONFIG_SMALL("PC Paintbrush PCX image"),
+    CODEC_LONG_NAME("PC Paintbrush PCX image"),
     .p.type         = AVMEDIA_TYPE_VIDEO,
     .p.id           = AV_CODEC_ID_PCX,
+    .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_ENCODER_REORDERED_OPAQUE,
     FF_CODEC_ENCODE_CB(pcx_encode_frame),
     .p.pix_fmts     = (const enum AVPixelFormat[]){
         AV_PIX_FMT_RGB24,
@@ -205,5 +206,4 @@ const FFCodec ff_pcx_encoder = {
         AV_PIX_FMT_MONOBLACK,
         AV_PIX_FMT_NONE
     },
-    .caps_internal  = FF_CODEC_CAP_INIT_THREADSAFE,
 };
