@@ -21,7 +21,7 @@
 #include "libavutil/attributes.h"
 #include "libavcodec/videodsp.h"
 
-static void prefetch_ppc(uint8_t *mem, ptrdiff_t stride, int h)
+static void prefetch_ppc(const uint8_t *mem, ptrdiff_t stride, int h)
 {
     register const uint8_t *p = mem;
     do {
