@@ -129,18 +129,12 @@ int main(int argc, char **argv)
     printf("ffmpeg motion test\n");
 
     ctx = avcodec_alloc_context3(NULL);
-    if (!ctx) {
-        return 1;
-    }
-
     ctx->flags |= AV_CODEC_FLAG_BITEXACT;
     av_force_cpu_flags(0);
-    memset(&cctx, 0, sizeof(cctx));
     ff_me_cmp_init(&cctx, ctx);
     for (c = 0; c < flags_size; c++) {
         int x;
         av_force_cpu_flags(flags[c]);
-        memset(&mmxctx, 0, sizeof(mmxctx));
         ff_me_cmp_init(&mmxctx, ctx);
 
         for (x = 0; x < 2; x++) {
