@@ -29,7 +29,9 @@
 #include "libavutil/time.h"
 
 #include "libavformat/avformat.h"
+#include "libavformat/demux.h"
 #include "libavformat/internal.h"
+#include "libavformat/version.h"
 #include "pulse_audio_common.h"
 #include "timefilter.h"
 
@@ -161,7 +163,12 @@ static av_cold int pulse_read_header(AVFormatContext *s)
         return AVERROR(ENOMEM);
     }
 
-    attr.fragsize = pd->fragment_size;
+    if (pd->fragment_size == -1) {
+        // 50 ms fragments/latency by default seem good enough
+        attr.fragsize = pa_frame_size(&ss) * (pd->sample_rate / 20);
+    } else {
+        attr.fragsize = pd->fragment_size;
+    }
 
     if (s->url[0] != '\0' && strcmp(s->url, "default"))
         device = s->url;
@@ -219,7 +226,7 @@ static av_cold int pulse_read_header(AVFormatContext *s)
 
     ret = pa_stream_connect_record(pd->stream, device, &attr,
                                     PA_STREAM_INTERPOLATE_TIMING
-                                    | (pd->fragment_size == -1 ? PA_STREAM_ADJUST_LATENCY : 0)
+                                    |PA_STREAM_ADJUST_LATENCY
                                     |PA_STREAM_AUTO_TIMING_UPDATE);
 
     if (ret < 0) {
@@ -259,7 +266,7 @@ static av_cold int pulse_read_header(AVFormatContext *s)
     st->codecpar->codec_type  = AVMEDIA_TYPE_AUDIO;
     st->codecpar->codec_id    = codec_id;
     st->codecpar->sample_rate = pd->sample_rate;
-    st->codecpar->channels    = pd->channels;
+    st->codecpar->ch_layout.nb_channels = pd->channels;
     avpriv_set_pts_info(st, 64, 1, 1000000);  /* 64 bits pts in us */
 
     pd->timefilter = ff_timefilter_new(1000000.0 / pd->sample_rate,
@@ -365,6 +372,7 @@ static int pulse_get_device_list(AVFormatContext *h, AVDeviceInfoList *device_li
 
 #define OFFSET(a) offsetof(PulseData, a)
 #define D AV_OPT_FLAG_DECODING_PARAM
+#define DEPR AV_OPT_FLAG_DEPRECATED
 
 static const AVOption options[] = {
     { "server",        "set PulseAudio server",                             OFFSET(server),        AV_OPT_TYPE_STRING, {.str = NULL},     0, 0, D },
@@ -372,7 +380,7 @@ static const AVOption options[] = {
     { "stream_name",   "set stream description",                            OFFSET(stream_name),   AV_OPT_TYPE_STRING, {.str = "record"}, 0, 0, D },
     { "sample_rate",   "set sample rate in Hz",                             OFFSET(sample_rate),   AV_OPT_TYPE_INT,    {.i64 = 48000},    1, INT_MAX, D },
     { "channels",      "set number of audio channels",                      OFFSET(channels),      AV_OPT_TYPE_INT,    {.i64 = 2},        1, INT_MAX, D },
-    { "frame_size",    "set number of bytes per frame",                     OFFSET(frame_size),    AV_OPT_TYPE_INT,    {.i64 = 1024},     1, INT_MAX, D },
+    { "frame_size",    "set number of bytes per frame",                     OFFSET(frame_size),    AV_OPT_TYPE_INT,    {.i64 = 1024},     1, INT_MAX, D | DEPR },
     { "fragment_size", "set buffering size, affects latency and cpu usage", OFFSET(fragment_size), AV_OPT_TYPE_INT,    {.i64 = -1},      -1, INT_MAX, D },
     { "wallclock",     "set the initial pts using the current time",     OFFSET(wallclock),     AV_OPT_TYPE_INT,    {.i64 = 1},       -1, 1, D },
     { NULL },
@@ -386,14 +394,14 @@ static const AVClass pulse_demuxer_class = {
     .category       = AV_CLASS_CATEGORY_DEVICE_AUDIO_INPUT,
 };
 
-AVInputFormat ff_pulse_demuxer = {
-    .name           = "pulse",
-    .long_name      = NULL_IF_CONFIG_SMALL("Pulse audio input"),
+const FFInputFormat ff_pulse_demuxer = {
+    .p.name          = "pulse",
+    .p.long_name     = NULL_IF_CONFIG_SMALL("Pulse audio input"),
+    .p.flags         = AVFMT_NOFILE,
+    .p.priv_class    = &pulse_demuxer_class,
     .priv_data_size = sizeof(PulseData),
     .read_header    = pulse_read_header,
     .read_packet    = pulse_read_packet,
     .read_close     = pulse_close,
     .get_device_list = pulse_get_device_list,
-    .flags          = AVFMT_NOFILE,
-    .priv_class     = &pulse_demuxer_class,
 };
