@@ -22,7 +22,8 @@
 #include "libavutil/opt.h"
 #include "vulkan_filter.h"
 #include "vulkan_spirv.h"
-#include "internal.h"
+
+#include "filters.h"
 #include "video.h"
 
 typedef struct ChromaticAberrationVulkanContext {
