@@ -26,6 +26,8 @@ void ff_fcmul_add_sse3(float *sum, const float *t, const float *c,
                        ptrdiff_t len);
 void ff_fcmul_add_avx(float *sum, const float *t, const float *c,
                       ptrdiff_t len);
+void ff_fcmul_add_fma3(float *sum, const float *t, const float *c,
+                       ptrdiff_t len);
 
 av_cold void ff_afir_init_x86(AudioFIRDSPContext *s)
 {
@@ -37,4 +39,7 @@ av_cold void ff_afir_init_x86(AudioFIRDSPContext *s)
     if (EXTERNAL_AVX_FAST(cpu_flags)) {
         s->fcmul_add = ff_fcmul_add_avx;
     }
+    if (EXTERNAL_FMA3_FAST(cpu_flags)) {
+        s->fcmul_add = ff_fcmul_add_fma3;
+    }
 }
