@@ -19,17 +19,57 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "libavutil/intreadwrite.h"
 #include "avformat.h"
+#include "demux.h"
 #include "internal.h"
 
 static int threedostr_probe(const AVProbeData *p)
 {
-    if (memcmp(p->buf, "CTRL", 4) &&
-        memcmp(p->buf, "SHDR", 4) &&
-        memcmp(p->buf, "SNDS", 4))
-        return 0;
+    for (int i = 0; i < p->buf_size;) {
+        unsigned chunk = AV_RL32(p->buf + i);
+        unsigned size  = AV_RB32(p->buf + i + 4);
 
-    return AVPROBE_SCORE_MAX / 3 * 2;
+        if (size < 8 || p->buf_size - i < size)
+            return 0;
+        i += 8;
+        size -= 8;
+        switch (chunk) {
+        case MKTAG('C','T','R','L'):
+            break;
+        case MKTAG('S','N','D','S'):
+            if (size < 56)
+                return 0;
+            i += 8;
+            if (AV_RL32(p->buf + i) != MKTAG('S','H','D','R'))
+                return 0;
+            i += 28;
+
+            if (AV_RB32(p->buf + i) <= 0)
+                return 0;
+            i += 4;
+            if (AV_RB32(p->buf + i) <= 0)
+                return 0;
+            i += 4;
+            if (AV_RL32(p->buf + i) == MKTAG('S','D','X','2'))
+                return AVPROBE_SCORE_MAX;
+            else
+                return 0;
+            break;
+        case MKTAG('S','H','D','R'):
+            if (size > 0x78) {
+                i += 0x78;
+                size -= 0x78;
+            }
+            break;
+        default:
+            break;
+        }
+
+        i += size;
+    }
+
+    return 0;
 }
 
 static int threedostr_read_header(AVFormatContext *s)
@@ -63,15 +103,15 @@ static int threedostr_read_header(AVFormatContext *s)
 
             st->codecpar->codec_type  = AVMEDIA_TYPE_AUDIO;
             st->codecpar->sample_rate = avio_rb32(s->pb);
-            st->codecpar->channels    = avio_rb32(s->pb);
-            if (st->codecpar->channels <= 0 || st->codecpar->sample_rate <= 0)
+            st->codecpar->ch_layout.nb_channels = avio_rb32(s->pb);
+            if (st->codecpar->ch_layout.nb_channels <= 0 || st->codecpar->sample_rate <= 0)
                 return AVERROR_INVALIDDATA;
             codec                  = avio_rl32(s->pb);
             avio_skip(s->pb, 4);
             if (ctrl_size == 20 || ctrl_size == 3 || ctrl_size == -1)
-                st->duration       = (avio_rb32(s->pb) - 1) / st->codecpar->channels;
+                st->duration       = (avio_rb32(s->pb) - 1) / st->codecpar->ch_layout.nb_channels;
             else
-                st->duration       = avio_rb32(s->pb) * 16 / st->codecpar->channels;
+                st->duration       = avio_rb32(s->pb) * 16 / st->codecpar->ch_layout.nb_channels;
             size -= 56;
             found_shdr = 1;
             break;
@@ -96,7 +136,7 @@ static int threedostr_read_header(AVFormatContext *s)
     switch (codec) {
     case MKTAG('S','D','X','2'):
         st->codecpar->codec_id    = AV_CODEC_ID_SDX2_DPCM;
-        st->codecpar->block_align = 1 * st->codecpar->channels;
+        st->codecpar->block_align = 1 * st->codecpar->ch_layout.nb_channels;
         break;
     default:
         avpriv_request_sample(s, "codec %X", codec);
@@ -110,15 +150,12 @@ static int threedostr_read_header(AVFormatContext *s)
 
 static int threedostr_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    unsigned chunk, size, found_ssmp = 0;
+    unsigned chunk, size;
     AVStream *st = s->streams[0];
     int64_t pos;
     int ret = 0;
 
-    while (!found_ssmp) {
-        if (avio_feof(s->pb))
-            return AVERROR_EOF;
-
+    while (!avio_feof(s->pb)) {
         pos   = avio_tell(s->pb);
         chunk = avio_rl32(s->pb);
         size  = avio_rb32(s->pb);
@@ -142,10 +179,8 @@ static int threedostr_read_packet(AVFormatContext *s, AVPacket *pkt)
             ret = av_get_packet(s->pb, pkt, size);
             pkt->pos = pos;
             pkt->stream_index = 0;
-            pkt->duration = size / st->codecpar->channels;
-            size = 0;
-            found_ssmp = 1;
-            break;
+            pkt->duration = size / st->codecpar->ch_layout.nb_channels;
+            return ret;
         default:
             av_log(s, AV_LOG_DEBUG, "skipping unknown chunk: %X\n", chunk);
             break;
@@ -154,15 +189,15 @@ static int threedostr_read_packet(AVFormatContext *s, AVPacket *pkt)
         avio_skip(s->pb, size);
     }
 
-    return ret;
+    return AVERROR_EOF;
 }
 
-AVInputFormat ff_threedostr_demuxer = {
-    .name           = "3dostr",
-    .long_name      = NULL_IF_CONFIG_SMALL("3DO STR"),
+const FFInputFormat ff_threedostr_demuxer = {
+    .p.name         = "3dostr",
+    .p.long_name    = NULL_IF_CONFIG_SMALL("3DO STR"),
+    .p.extensions   = "str",
+    .p.flags        = AVFMT_GENERIC_INDEX,
     .read_probe     = threedostr_probe,
     .read_header    = threedostr_read_header,
     .read_packet    = threedostr_read_packet,
-    .extensions     = "str",
-    .flags          = AVFMT_GENERIC_INDEX,
 };
