@@ -18,7 +18,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "libavutil/channel_layout.h"
 #include "internal.h"
+#include "version.h"
 
 int ff_framehash_write_header(AVFormatContext *s)
 {
@@ -34,12 +36,14 @@ int ff_framehash_write_header(AVFormatContext *s)
         avio_printf(s->pb, "#media_type %d: %s\n", i, av_get_media_type_string(avctx->codec_type));
         avio_printf(s->pb, "#codec_id %d: %s\n", i, avcodec_get_name(avctx->codec_id));
         switch (avctx->codec_type) {
-        case AVMEDIA_TYPE_AUDIO:
-            av_get_channel_layout_string(buf, sizeof(buf), avctx->channels, avctx->channel_layout);
+        case AVMEDIA_TYPE_AUDIO: {
+            int ret = av_channel_layout_describe(&avctx->ch_layout, buf, sizeof(buf));
+            if (ret < 0)
+                return ret;
             avio_printf(s->pb, "#sample_rate %d: %d\n", i,avctx->sample_rate);
-            avio_printf(s->pb, "#channel_layout %d: %"PRIx64"\n", i,avctx->channel_layout);
             avio_printf(s->pb, "#channel_layout_name %d: %s\n", i, buf);
             break;
+            }
         case AVMEDIA_TYPE_VIDEO:
             avio_printf(s->pb, "#dimensions %d: %dx%d\n", i, avctx->width, avctx->height);
             avio_printf(s->pb, "#sar %d: %d/%d\n", i, st->sample_aspect_ratio.num, st->sample_aspect_ratio.den);
