@@ -19,11 +19,12 @@
 #include "config.h"
 #include "libavutil/avassert.h"
 #include "libavutil/avstring.h"
+#include "libavutil/mem.h"
 
 #include "libavcodec/avcodec.h"
 #include "libavcodec/bytestream.h"
 #include "libavformat/avformat.h"
-
+#include "libavformat/demux.h"
 
 typedef struct IOContext {
     int64_t pos;
@@ -34,6 +35,13 @@ typedef struct IOContext {
 
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size);
 
+int64_t interrupt_counter;
+static int interrupt_cb(void *ctx)
+{
+    interrupt_counter --;
+    return interrupt_counter < 0;
+}
+
 static void error(const char *err)
 {
     fprintf(stderr, "%s", err);
@@ -106,12 +114,12 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
     static int c;
     int seekable = 0;
     int ret;
-    AVInputFormat *fmt = NULL;
+    const AVInputFormat *fmt = NULL;
 #ifdef FFMPEG_DEMUXER
 #define DEMUXER_SYMBOL0(DEMUXER) ff_##DEMUXER##_demuxer
 #define DEMUXER_SYMBOL(DEMUXER) DEMUXER_SYMBOL0(DEMUXER)
-    extern AVInputFormat DEMUXER_SYMBOL(FFMPEG_DEMUXER);
-    fmt = &DEMUXER_SYMBOL(FFMPEG_DEMUXER);
+    extern const FFInputFormat DEMUXER_SYMBOL(FFMPEG_DEMUXER);
+    fmt = &DEMUXER_SYMBOL(FFMPEG_DEMUXER).p;
 #endif
 
     if (!c) {
@@ -160,8 +168,15 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
                 *strchr(extension, ',') = 0;
             av_strlcatf(filename, sizeof(filename), ".%s", extension);
         }
+
+        interrupt_counter = bytestream2_get_le32(&gbc);
+        avfmt->interrupt_callback.callback = interrupt_cb;
     }
 
+    // HLS uses a loop with sleep, we thus must breakout or we timeout
+    if (fmt && !strcmp(fmt->name, "hls"))
+        interrupt_counter &= 31;
+
     if (!io_buffer_size || size / io_buffer_size > maxblocks)
         io_buffer_size = size;
 
